using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Data;
using System.IO;
using System.Xml;
using System.Text;

namespace BeerTaster
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
        private System.Windows.Forms.MainMenu mainMenu;
        private System.Windows.Forms.MenuItem toolsMenuItem;
        private System.Windows.Forms.DataGrid dataGrid;
        private System.Windows.Forms.ContextMenu contextMenu;
        private System.Windows.Forms.MenuItem deleteMenuItem;
        private System.Windows.Forms.MenuItem bluetoothMenuItem;
        private System.Windows.Forms.MenuItem menuItem2;

        private DataSet beerDataSet = new DataSet();
        private System.Windows.Forms.MenuItem addMenuItem;
        private string fileName = @"\Program Files\BeerTaster\BeerData.xml";

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

            this.Activated += new EventHandler(Form1_Activated);
            this.Deactivate += new EventHandler(Form1_Deactivate);
        
            // Assigning this at design time causes an error - don't know why
            dataGrid.ContextMenu = contextMenu;
        }
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.mainMenu = new System.Windows.Forms.MainMenu();
            this.toolsMenuItem = new System.Windows.Forms.MenuItem();
            this.addMenuItem = new System.Windows.Forms.MenuItem();
            this.dataGrid = new System.Windows.Forms.DataGrid();
            this.contextMenu = new System.Windows.Forms.ContextMenu();
            this.bluetoothMenuItem = new System.Windows.Forms.MenuItem();
            this.menuItem2 = new System.Windows.Forms.MenuItem();
            this.deleteMenuItem = new System.Windows.Forms.MenuItem();
            // 
            // mainMenu
            // 
            this.mainMenu.MenuItems.Add(this.toolsMenuItem);
            // 
            // toolsMenuItem
            // 
            this.toolsMenuItem.MenuItems.Add(this.addMenuItem);
            this.toolsMenuItem.Text = "Tools";
            // 
            // addMenuItem
            // 
            this.addMenuItem.Text = "Add New Beer";
            this.addMenuItem.Click += new System.EventHandler(this.addMenuItem_Click);
            // 
            // dataGrid
            // 
            this.dataGrid.Size = new System.Drawing.Size(240, 268);
            this.dataGrid.Click += new System.EventHandler(this.dataGrid_Click);
            // 
            // contextMenu
            // 
            this.contextMenu.MenuItems.Add(this.bluetoothMenuItem);
            this.contextMenu.MenuItems.Add(this.menuItem2);
            this.contextMenu.MenuItems.Add(this.deleteMenuItem);
            // 
            // bluetoothMenuItem
            // 
            this.bluetoothMenuItem.Text = "Send via Bluetooth";
            this.bluetoothMenuItem.Click += new System.EventHandler(this.bluetoothMenuItem_Click);
            // 
            // menuItem2
            // 
            this.menuItem2.Text = "-";
            // 
            // deleteMenuItem
            // 
            this.deleteMenuItem.Text = "Delete Beer";
            this.deleteMenuItem.Click += new System.EventHandler(this.deleteMenuItem_Click);
            // 
            // Form1
            // 
            this.Controls.Add(this.dataGrid);
            this.Menu = this.mainMenu;
            this.Text = "Beer Taster";
            this.Load += new System.EventHandler(this.Form1_Load);

        }
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>

		static void Main() 
		{
			Application.Run(new Form1());
		}


        private void CreateDataSet()
        {
            DataTable beerTable = new DataTable("BeerTable");

            // Add columns
            beerTable.Columns.Add("Name",typeof(string));
            beerTable.Columns.Add("Country",typeof(string));
            beerTable.Columns.Add("Type",typeof(string));
            beerTable.Columns.Add("Rating",typeof(int));
            beerDataSet.Tables.Add(beerTable);
        }

        private void AddBeerTheHardWay(string name,string country,string type,int rating)
        {
            // More verbose, but clearer
            DataTable beerTable = beerDataSet.Tables["BeerTable"];
            DataRow newRow = beerTable.NewRow();

            newRow["Name"] = name;
            newRow["Country"] = country;
            newRow["Type"] = type;
            newRow["Rating"] = rating;

            beerTable.Rows.Add(newRow);
        }

        private void AddBeerTheEasyWay(params object[] beerValues)
        {
            // Shorter code, but not as obvious what order parameters should be etc
            DataTable beerTable = beerDataSet.Tables["BeerTable"];
            beerTable.Rows.Add(beerValues);
        }


        private void FillDataSet()
        {
            AddBeerTheHardWay("Hoegaarden","Belgian","White",4);
            AddBeerTheHardWay("Cooper's Ale","Australian","Lager",3);
            AddBeerTheHardWay("Westvleteren 8","Belgian","Trappist",5);
            AddBeerTheHardWay("Nastra Azzuro","Italian","Lager",3);
            AddBeerTheHardWay("Brakspears Ordinary","English","Ale",2);
        }

        private void DeleteRow(int rowIndex)
        {
            DataTable beerTable = beerDataSet.Tables["BeerTable"];

            // This is an unrecoverable operation (at least the way I'm doing it here)
            // so make sure the user really wants to do this
            if (MessageBox.Show("Are you sure you want to delete " + beerTable.Rows[rowIndex]["Name"].ToString() + "?", 
                                "Deleting Beer",
                                MessageBoxButtons.YesNo,MessageBoxIcon.Question,
                                MessageBoxDefaultButton.Button1) == DialogResult.No)
            {
                return;
            }


            beerTable.Rows[rowIndex].Delete();
            // Row is not actually deleted yet, so commit changes
            // Can commit changes for row, table or dataset
            beerDataSet.AcceptChanges();
        }

        private void LoadData()
        {
            if (File.Exists(fileName)) 
            {
                beerDataSet.ReadXml(fileName);
            } 
            else 
            {
                CreateDataSet();
                FillDataSet();
            }
        }

        private void SaveData()
        {
            beerDataSet.WriteXml(fileName);
        }

        private void Form1_Load(object sender, System.EventArgs e)
        {
            LoadData();
            dataGrid.DataSource = beerDataSet.Tables["BeerTable"];
        }

        private void addMenuItem_Click(object sender, System.EventArgs e)
        {
            // Use rowIndex < 0 to indicate need to create a new row
            EditForm editForm = new EditForm(beerDataSet.Tables["BeerTable"],-1);

            editForm.Show();
        }

        private void dataGrid_Click(object sender, System.EventArgs e)
        {
            // Tapping on a row should select the row and show a form to edit the corresponding record
            //int currRowIndex = dataGrid.HitTest(MousePosition.X,MousePosition.Y).Row;

            int currRowIndex = dataGrid.CurrentRowIndex;

            if (currRowIndex < 0) 
            {
                // Tapped on the header row, so no need to do anything
                return;
            }

            //currRowIndex--;
            dataGrid.Select(currRowIndex);

            EditForm editForm = new EditForm(beerDataSet.Tables["BeerTable"],currRowIndex);

            editForm.Show();
        }

        private void bluetoothMenuItem_Click(object sender, System.EventArgs e)
        {
            MessageBox.Show("You're dreaming if you think I'm going to implement this!");
        }

        private void deleteMenuItem_Click(object sender, System.EventArgs e)
        {
            if (dataGrid.CurrentRowIndex >= 0) 
            {
                // Subtract one because row 0 is the header row
                DeleteRow(dataGrid.CurrentRowIndex);
            }
        }

        private void Form1_Activated(object sender, EventArgs e)
        {
            // Could read data here, reopen dataset and so on
        }

        private void Form1_Deactivate(object sender, EventArgs e)
        {
            // Save data here  
            SaveData();
        }
    }
}
